#include "ac33mx128_pcu.h"
#include "ac33mx128_scu.h"
#include "ac33mx128_gpio.h"
#include "ac33mx128_i2c.h"
#include "debug_frmwrk.h"
#include "ac33mx128_libcfg.h"

// system default I/O setting
void PCU_Init(void);

#define M24C02_ADDR    (0xA0>>1)
/** Own Slave address in Slave I2C device */
#define I2CDEV_S_ADDR	(0xC0>>1)

/** Max buffer length */
#define BUFFER_SIZE			10

Bool complete;
Bool isMasterMode = TRUE;
/** These global variables below used in interrupt mode - Slave device ----------------*/
uint8_t Master_Buf[BUFFER_SIZE];

const uint8_t menu[] =
"\n\r************************************************\n\r"
"I2C demo \n\r"
"\t - MCU: AC33M8128 \n\r"
"\t - Core: ARM Cortex-M3 \n\r"
"\t - Communicate via: UART0 - 115200 bps \n\r"
"\n\rPress 1-4 to select I2C running mode:\n\r\n\r"
"\t 1 - I2C Polling   Master\n\r"
"\t 2 - I2C Interrupt Master\n\r"
"\t 3 - I2C Polling   Slave \n\r"
"\t 4 - I2C Interrupt Slave \n\r"
"************************************************\n\r";
const uint8_t  menu1[] = "\n\r \t - Press x to exit this mode!\n\r";
const uint8_t  menu3[] = "\t - Press c to continue...\n\r";

void I2C0_IRQHandler(void)
{
	if (isMasterMode)// Master Mode
	{
		I2C_Interrupt_MasterHandler(IC0);
		if (I2C_MasterTransferComplete(IC0)){
			complete = TRUE;
		}
	}
	else// Slave Mode
	{
		I2C_Interrupt_SlaveHandler(IC0);
		if (I2C_SlaveTransferComplete(IC0))
		{
			complete = TRUE;
		}
	}	
}

void I2C1_IRQHandler(void)
{
	if (isMasterMode)// Master Mode
	{
		I2C_Interrupt_MasterHandler(IC1);
		if (I2C_MasterTransferComplete(IC1)){
			complete = TRUE;
		}
	}
	else// Slave Mode
	{
		I2C_Interrupt_SlaveHandler(IC1);
		if (I2C_SlaveTransferComplete(IC1))
		{
			complete = TRUE;
		}
	}	
}

/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void print_menu(void)
{
	_DBG(menu);
}

/*********************************************************************//**
 * @brief		Initialize buffer
 * @param[in]	buffer	buffer to initialize
 * @param[in]	type:
 * 				- 0: Initialize buffer with 0
 * 				- 1: Initialize buffer with increment value from 0
 * @return 		None
 **********************************************************************/
void Buffer_Init(uint8_t* buffer, uint8_t type)
{
	uint8_t i;

	if (type)
	{
		for (i = 0; i < BUFFER_SIZE; i++) {
			buffer[i] = i;
		}
	}
	else
	{
		for (i = 0; i < BUFFER_SIZE; i++) {
			buffer[i] = 0;
		}
	}
}

/*********************************************************************//**
 * @brief		I2C master application in polling mode
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void App_I2C_Polling_Master(void)
{
	I2C_M_SETUP_Type transferMCfg;
	uint8_t buffer = 0xFF, regAddr = 0x00;
	uint8_t Master_Buf[BUFFER_SIZE];

	while (1) {
		_DBG_("\n\r\n\rI2C Polling Master is running...\n\r");
		_DBG_(menu1);
		_DBG_(menu3);

		while (1)
		{
			buffer = _DG;
			if (buffer == 'x' || buffer == 'X')
			{
				return;
			}
			else if (buffer == 'c' || buffer == 'C')
			{
				break;
			}
		}
		/* Transmit -------------------------------------------------------- */
		_DBG_("\n\rPress '1' to transmit");
		while (1)
		{
			buffer = _DG;
			if (buffer == '1')
				break;
		}
		_DBG_("\n\rTransmitting...");

		/* Initialize buffer */
		Buffer_Init(Master_Buf, 0);
		Master_Buf[0]=0x00; //address
		Master_Buf[1]=0xa2; //data	
		Master_Buf[2]=0xa4; //data	
		Master_Buf[3]=0xa6; //data	
		Master_Buf[4]=0xa8; //data		
		Master_Buf[5]=0xaA; //data		
		transferMCfg.sl_addr7bit = M24C02_ADDR;
		transferMCfg.tx_data = Master_Buf;
		transferMCfg.tx_length = 6;
		I2C_MasterTransmitData(IC0, &transferMCfg, I2C_TRANSFER_POLLING);

		/* Transmit and receive -------------------------------------------------------- */
		_DBG_("\n\rPress '2' to Transmit, then repeat start and receive...");
		while (1)
		{
			buffer = _DG;
			if (buffer == '2')
				break;
		}

		/* Initialize buffer */
		Buffer_Init(Master_Buf, 0);

		transferMCfg.sl_addr7bit = M24C02_ADDR;
		transferMCfg.tx_data = &regAddr;
		transferMCfg.tx_length = 1;
		transferMCfg.rx_data = Master_Buf;
		transferMCfg.rx_length = 5;
		transferMCfg.tx_count = 0;
		transferMCfg.rx_count = 0;
		I2C_MasterTransferData(IC0, &transferMCfg, I2C_TRANSFER_POLLING);
#ifdef  __CC_ARM
		__nop();		// Use only keil arm compiler
#elif	__ICCARM__
		asm("nop");	// Use only iar arm compiler
#endif	
	}
}

/***********************************************************************
 * @brief		I2C master application using interrupt
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void App_I2C_Interrupt_Master(void)
{
	I2C_M_SETUP_Type transferMCfg;
	uint8_t buffer = 0xFF, regAddr = 0;
	uint8_t Master_Buf[BUFFER_SIZE];

	while (1) {
		_DBG_("\n\r\n\rI2C Interrupt Master is running...\n\r");
		_DBG_(menu1);
		_DBG_(menu3);

		while (1)
		{
			buffer = _DG;
			if (buffer == 'x' || buffer == 'X')
			{
				return;
			}
			else if (buffer == 'c' || buffer == 'C')
			{
				break;
			}
		}
		/* Transmit -------------------------------------------------------- */
		_DBG_("\n\rPress '1' to transmit");
		while (1)
		{
			buffer = _DG;
			if (buffer == '1')
				break;
		}
		_DBG_("\n\rTransmitting...");

		/* Initialize buffer */
		complete = FALSE;
		Buffer_Init(Master_Buf, 0);
		Master_Buf[0]=0x00; //address
		Master_Buf[1]=0x2A; //data	
		Master_Buf[2]=0x4A; //data	
		Master_Buf[3]=0x6A; //data	
		Master_Buf[4]=0x8A; //data		
		Master_Buf[5]=0xA5; //data		
		transferMCfg.sl_addr7bit = M24C02_ADDR;
		transferMCfg.tx_data = Master_Buf;
		transferMCfg.tx_length = 6;
		I2C_MasterTransmitData(IC0, &transferMCfg, I2C_TRANSFER_INTERRUPT);
		while(!complete);

		/* Transmit and receive -------------------------------------------------------- */
		_DBG_("\n\rPress '2' to Transmit, then repeat start and receive...");
		while (1)
		{
			buffer = _DG;
			if (buffer == '2')
				break;
		}

		/* Initialize buffer */
		complete = FALSE;
		Buffer_Init(Master_Buf, 0);
		transferMCfg.sl_addr7bit = M24C02_ADDR;
		transferMCfg.tx_data = &regAddr;
		transferMCfg.tx_length = 1;
		transferMCfg.rx_data = Master_Buf;
		transferMCfg.rx_length = 5;
		transferMCfg.tx_count = 0;
		transferMCfg.rx_count = 0;
		
		I2C_MasterTransferData(IC0, &transferMCfg, I2C_TRANSFER_INTERRUPT);
		while(!complete);
#ifdef  __CC_ARM
		__nop();		// Use only keil arm compiler
#elif	__ICCARM__
		asm("nop");	// Use only iar arm compiler
#endif	
	}
}

/**********************************************************************
 * @brief		I2C slave application in polling mode
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void App_I2C_Polling_Slave (void)
{
	I2C_S_SETUP_Type transferSCfg;
	uint8_t buffer = 0xFF;
	uint8_t Slave_Buf[BUFFER_SIZE], S_buff[2];

	while (1) {
		_DBG_("\n\r\n\rI2C Polling Slave is running...\n\r");
		_DBG_(menu1);
		_DBG_(menu3);

		while (1)
		{
			buffer = _DG;
			if (buffer == 'x' || buffer == 'X')
			{
				return;
			}
			else if (buffer == 'c' || buffer == 'C')
			{
				break;
			}
		}
		/* Receive -------------------------------------------------------- */
		_DBG_("\n\rPress '1' to Start");

		while (1)
		{
			buffer = _DG;
			if (buffer == '1')
				break;
		}
		_DBG_("\n\rReceiving...");

		/* Initialize buffer */
		Buffer_Init(Slave_Buf, 0);

		transferSCfg.rx_data = Slave_Buf;
		transferSCfg.rx_length = 3;
		I2C_SlaveReceiveData(IC1, &transferSCfg, I2C_TRANSFER_POLLING);
#ifdef  __CC_ARM
		__nop();		// Use only keil arm compiler
#elif	__ICCARM__
		asm("nop");	// Use only iar arm compiler
#endif	

		/* Receive and transmit -------------------------------------------------------- */
		_DBG_("\n\rStart Receive, wait for repeat start and transmit...");

		/* Initialize buffer */
		Buffer_Init(Slave_Buf, 1);

		transferSCfg.rx_data = S_buff;
		transferSCfg.rx_length = 2;
		Slave_Buf[0]=0x8A; //data	
		Slave_Buf[1]=0xAA; //data		
		Slave_Buf[2]=0xA5; //data	
		transferSCfg.tx_data = Slave_Buf;
		transferSCfg.tx_length = 3;
		I2C_SlaveTransferData(IC1, &transferSCfg, I2C_TRANSFER_POLLING);
#ifdef  __CC_ARM
		__nop();		// Use only keil arm compiler
#elif	__ICCARM__
		asm("nop");	// Use only iar arm compiler
#endif			
	}
}

/*********************************************************************//**
 * @brief		I2C slave application using interrupt
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void App_I2C_Interrupt_Slave (void)
{
	I2C_S_SETUP_Type transferSCfg;
	uint8_t buffer = 0xFF;
	uint8_t Slave_Buf[BUFFER_SIZE], S_buff[2];

	while (1) {
		_DBG_("\n\r\n\rI2C Interrupt Slave is running...\n\r");
		_DBG_(menu1);
		_DBG_(menu3);

		while (1)
		{
			buffer = _DG;
			if (buffer == 'x' || buffer == 'X')
			{
				return;
			}
			else if (buffer == 'c' || buffer == 'C')
			{
				break;
			}
		}
		/* Receive -------------------------------------------------------- */
		_DBG_("\n\rPress '1' to Start");

		while (1)
		{
			buffer = _DG;
			if (buffer == '1')
				break;
		}
		_DBG_("\n\rReceiving...");

		/* Initialize buffer */
		Buffer_Init(Slave_Buf, 0);

		complete = FALSE;
		transferSCfg.rx_data = Slave_Buf;
		transferSCfg.rx_length = 3;
		I2C_SlaveReceiveData(IC1, &transferSCfg, I2C_TRANSFER_INTERRUPT);
		 while(!complete);
#ifdef  __CC_ARM
		__nop();		// Use only keil arm compiler
#elif	__ICCARM__
		asm("nop");	// Use only iar arm compiler
#endif	
		
		/* Receive and transmit -------------------------------------------------------- */
		_DBG_("\n\rStart Receive, wait for repeat start and transmit...");

		/* Initialize buffer */
		Buffer_Init(Slave_Buf, 0);

		complete = FALSE;
		
		transferSCfg.rx_data = S_buff;
		transferSCfg.rx_length = 2;
		Slave_Buf[0]=0x6A; //data	
		Slave_Buf[1]=0x8A; //data		
		Slave_Buf[2]=0xA5; //data		
		transferSCfg.tx_data = Slave_Buf;
		transferSCfg.tx_length = 3;
		I2C_SlaveTransferData(IC1, &transferSCfg, I2C_TRANSFER_INTERRUPT);
		 while(!complete);
#ifdef  __CC_ARM
		__nop();		// Use only keil arm compiler
#elif	__ICCARM__
		asm("nop");	// Use only iar arm compiler
#endif	
	}
}

void mainloop(void)
{
	uint8_t buffer = 0xFF;
	
	// Test Pin setting PD9
	PCU_ConfigureFunction(PCD, 9, PD9_MUX_PD9);
	PCU_SetDirection(PCD, 9, PUSH_PULL_OUTPUT);
	PCU_ConfigurePullup(PCD, 9, DISABLE);	
	GPIO_SetValue(PD, _BIT(9));

	// I2C PC7:SCL0, PC8:SDA0	
	PCU_ConfigureFunction(PCC, 7, PC7_MUX_SCL0);
	PCU_SetDirection(PCC, 7, OPEN_DRAIN_OUTPUT);
	PCU_ConfigurePullup(PCC, 7, DISABLE);	

	PCU_ConfigureFunction(PCC, 8, PC8_MUX_SDA0);
	PCU_SetDirection(PCC, 8, OPEN_DRAIN_OUTPUT);
	PCU_ConfigurePullup(PCC, 8, DISABLE);	
	
	// I2C PD4:SCL1, PD5:SDA1	
	PCU_ConfigureFunction(PCD, 4, PD4_MUX_SCL1);
	PCU_SetDirection(PCD, 4, OPEN_DRAIN_OUTPUT);
	PCU_ConfigurePullup(PCD, 4, DISABLE);	

	PCU_ConfigureFunction(PCD, 5, PD5_MUX_SDA1);
	PCU_SetDirection(PCD, 5, OPEN_DRAIN_OUTPUT);
	PCU_ConfigurePullup(PCD, 5, DISABLE);	
	

	/* I2C block ------------------------------------------------------------------- */
	// Initialize Slave I2C peripheral
	I2C_Init(IC0, 100000);
	I2C_Init(IC1, 100000);

	/* Set  Own slave address for I2C1 device */
	I2C_SetOwnSlaveAddr(IC1, I2CDEV_S_ADDR, DISABLE);
	__enable_irq();

	print_menu();
	
	while (1) {
		buffer = _DG;
		
		switch (buffer)
		{
			case '1': //Master Polling Mode
				App_I2C_Polling_Master();
				print_menu();
				break;
			case '2': //Master Interrupt Mode
				isMasterMode = TRUE;
				App_I2C_Interrupt_Master();
				print_menu();
				break;
			case '3': //Slave Polling Mode
				App_I2C_Polling_Slave();
				print_menu();
				break;
			case '4': //Slave Interrupt Mode
				isMasterMode = FALSE;
				App_I2C_Interrupt_Slave();
				print_menu();
				break;
			default:
				break;
		}
	}
}

int main(void)
{
	SystemInit();
	PCU_Init(); // all port control setting  !!You have to set PORT function!!
	SCU_ClockInit(); // MCLK=72Mhz(PLL) setting 
	debug_frmwrk_init();
	
	mainloop();
	
	return (0);
}


void PCU_Init(void)
{
//Peripheral Enable Register 1  0:Disable, 1:Enable	
	SCU->PER1=SCU->PER1  
	| (1<<11)   // GPIOD
	| (1<<10)   // GPIOC
	| (1<<9)   // GPIOB
	| (1<<8)   // GPIOA
		; 		
//Peripheral Clock Enable Register 1 0:Disable, 1:Enable	
	SCU->PCER1=SCU->PCER1
	| (1<<11)   // GPIOD
	| (1<<10)   // GPIOC
	| (1<<9)   // GPIOB
	| (1<<8)   // GPIOA
		; 	
	
 	PORT_ACCESS_EN();  // enable writing permittion of ALL PCU register

	PCA->MR = 0 // PinMux setting (00b,01b,10b,11b)
	| (0<<30)   // 0:PA15, 1:MISO0, 2:T7C, 3:AN15
	| (0<<28)   // 0:PA14, 1:MOSI0,            3:AN14
	| (0<<26)   // 0:PA13, 1:SCK0,              3:AN13
	| (0<<24)   // 0:PA12, 1:SS0, 2:AD2I,     3:AN12
	| (0<<22)   // 0:PA11, 1:TRACED2,        3:AN11
	| (0<<20)   // 0:PA10, 1:TRACED1, 2:AD2O, 3:AN10
	| (0<<18)   // 0:PA9,  1:TRACED0, 2:AD1O, 3:AN9
	| (0<<16)   // 0:PA8, 1:TRACECLK, 2:AD0O, 3:AN8
	| (0<<14)   // 0:PA7, 1:TRACED3, 2:T3O, 3:AN7_CREF1
	| (0<<12)   // 0:PA6,                     2:T2O, 3:AN6_CREF0
	| (0<<10)   // 0:PA5,                     2:T1O, 3:AN5
	| (0<<8)   // 0:PA4,                       2:T0O, 3:AN4
	| (0<<6)   // 0:PA3,                            3:AN3_CP3
	| (0<<4)   // 0:PA2,                            3:AN2_CP2	
	| (0<<2)   // 0:PA1,                            3:AN1_CP1	
	| (0<<0)   // 0:PA0,                            3:AN0_CP0		
		; 
	PCA->CR = 0 // 0xFFFFFFFF(reset value) Input, output, open drain, analog
	| (0<<30)   // P15 0:Push-pull output, 1:Open-drain output, 2:Input, 3:Analog
	| (0<<28)   // P14
	| (0<<26)   // P13
	| (0<<24)   // P12
	| (0<<22)   // P11
	| (0<<20)   // P10
	| (0<<18)   // P9
	| (0<<16)   // P8
	| (0<<14)   // P7
	| (0<<12)   // P6
	| (0<<10)   // P5
	| (0<<8)   // P4
	| (0<<6)   // P3
	| (0<<4)   // P2
	| (0<<2)   // P1
	| (0<<0)   // P0	
		; 	
	PCA->PCR = 0 // pullup (0,1)
	| (0<<15)   // P15 0:Disable pull-up resistor, 1:Enable pull-up resister
	| (0<<14)   // P14
	| (0<<13)   // P13
	| (0<<12)   // P12
	| (0<<11)   // P11
	| (0<<10)   // P10
	| (0<<9)   // P9
	| (0<<8)   // P8
	| (0<<7)   // P7
	| (0<<6)   // P6
	| (0<<5)   // P5
	| (0<<4)   // P4
	| (0<<3)   // P3
	| (0<<2)   // P2
	| (0<<1)   // P1
	| (0<<0)   // P0	
		; 		
	
	
	PCB->MR = 0 // PinMux setting (00b,01b,10b,11b)
	| (0UL<<30)   // 0:PB15, 1:MP1WL
	| (0UL<<28)   // 0:PB14, 1:MP1WH
	| (0UL<<26)   // 0:PB13, 1:MP1VL
	| (0UL<<24)   // 0:PB12, 1:MP1VH
	| (0UL<<22)   // 0:PB11, 1:MP1UL
	| (0UL<<20)   // 0:PB10, 1:MP1UH
	| (0UL<<18)   // 0:PB9,  1:OVIN1, 2:TXD3
	| (0UL<<16)   // 0:PB8, 1:PRTIN1, 2:RXD3
	| (0UL<<14)   // 0:PB7, 1:OVIN0, 2:STBYO
	| (0UL<<12)   // 0:PB6, 1:PRTIN0, 2:WDTO
	| (0UL<<10)   // 0:PB5, 1:MP0WL, 2:T9O
	| (0UL<<8)   // 0:PB4, 1:MP0WH, 2:T9C
	| (0UL<<6)   // 0:PB3, 1:MP0VL
	| (0UL<<4)   // 0:PB2, 1:MP0VH
	| (0UL<<2)   // 0:PB1, 1:MP0UL
	| (0UL<<0)   // 0:PB0, 1:MP0UH 
		;	
	PCB->CR = 0   // 0xFFFFFFFF(reset value) Input, output, open drain, analog
	| (0UL<<30)   // P15 0:Push-pull output, 1:Open-drain output, 2:Input, 3:Analog
	| (0UL<<28)   // P14
	| (0UL<<26)   // P13
	| (0UL<<24)   // P12
	| (0UL<<22)   // P11
	| (0UL<<20)   // P10
	| (2UL<<18)   // P9
	| (2UL<<16)   // P8
	| (2UL<<14)   // P7
	| (2UL<<12)   // P6
	| (0UL<<10)   // P5
	| (0UL<<8)   // P4
	| (0UL<<6)   // P3
	| (0UL<<4)   // P2
	| (0UL<<2)   // P1
	| (0UL<<0)   // P0	
		; 	
	PCB->PCR = 0 // pullup (0,1)
	| (0<<15)   // P15 0:Disable pull-up resistor, 1:Enable pull-up resister
	| (0<<14)   // P14
	| (0<<13)   // P13
	| (0<<12)   // P12
	| (0<<11)   // P11
	| (0<<10)   // P10
	| (0<<9)   // P9
	| (0<<8)   // P8
	| (0<<7)   // P7
	| (0<<6)   // P6
	| (0<<5)   // P5
	| (0<<4)   // P4
	| (0<<3)   // P3
	| (0<<2)   // P2
	| (0<<1)   // P1
	| (0<<0)   // P0	
		; 
	
	PCC->MR = 0 //0x00100155(reset value) // PinMux setting (00b,01b,10b,11b)
	| (1<<30)   // 0:PC15, 1:TXD0, 2:MOSI0
	| (1<<28)   // 0:PC14, 1:RXD0, 2:MISO0, 3:VMARGIN
	| (1<<26)   // 0:PC13, 1:XOUT
	| (1<<24)   // 0:PC12, 1:XIN
	| (0<<22)   // 0:PC11,            2:T8C
	| (1<<20)   // 0:PC10, 1:nRESET,
	| (0<<18)   // 0:PC9,  1:CLKO, 2:P8O
	| (1<<16)   // 0:PC8, 1:SDA0
	| (1<<14)   // 0:PC7, 1:SCL0, 2:T3C
	| (0<<12)   // 0:PC6, 1:TXD1, 2:T2C_PHZ
	| (0<<10)   // 0:PC5, 1:RXD1, 2:T1C_PHB
	| (1<<8)   // 0:PC4, 1:nTRST, 2:T0C_PHA
	| (1<<6)   // 0:PC3, 1:TDI
	| (1<<4)   // 0:PC2, 1:TDO_SWO
	| (1<<2)   // 0:PC1, 1:TMS_SWDIO
	| (1<<0)   // 0:PC0, 1:TCK_SWCLK 
		;		
	PCC->CR = 0   // 0xFFAFFE8A(reset value) Input, output, open drain, analog
	| (0<<30)   // P15 0:Push-pull output, 1:Open-drain output, 2:Input, 3:Analog
	| (2<<28)   // P14
	| (3<<26)   // P13  if XOUT, it must be Analog 
	| (3<<24)   // P12  if XIN, it must be Analog
	| (0<<22)   // P11
	| (2<<20)   // P10
	| (0<<18)   // P9
	| (1<<16)   // P8
	| (1<<14)   // P7
	| (0<<12)   // P6
	| (0<<10)   // P5
	| (2<<8)   // P4
	| (2<<6)   // P3
	| (2<<4)   // P2
	| (2<<2)   // P1
	| (2<<0)   // P0	
		; 	
	PCC->PCR = 0 // pullup (0,1)
	| (0<<15)   // P15 0:Disable pull-up resistor, 1:Enable pull-up resister
	| (0<<14)   // P14
	| (0<<13)   // P13
	| (0<<12)   // P12
	| (0<<11)   // P11
	| (0<<10)   // P10
	| (0<<9)   // P9
	| (0<<8)   // P8
	| (0<<7)   // P7
	| (0<<6)   // P6
	| (0<<5)   // P5
	| (0<<4)   // P4
	| (0<<3)   // P3
	| (0<<2)   // P2
	| (0<<1)   // P1
	| (0<<0)   // P0	
		; 

	PCD->MR = 0 // PinMux setting (00b,01b,10b,11b)
	| (0<<30)   // 0:PD15, 1:AD2EOC
	| (0<<28)   // 0:PD14, 1:AD2SOC
	| (0<<26)   // 0:PD13, 1:AD1EOC, 2:T3C
	| (0<<24)   // 0:PD12, 1:AD1SOC, 2:T2C_PHZ
	| (0<<22)   // 0:PD11, 1:AD0EOC, 2:T1C_PHB
	| (0<<20)   // 0:PD10, 1:AD0SOC, 2:T0C_PHA
	| (0<<18)   // 0:PD9,                     2:STBYO
	| (0<<16)   // 0:PD8,                     2:WDTO
	| (0<<14)   // 0:PD7, 1:RXD2, 2:AD1I
	| (0<<12)   // 0:PD6, 1:TXD2, 2:AD0I
	| (1<<10)   // 0:PD5, 1:SDA1
	| (1<<8)   // 0:PD4, 1:SCL1
	| (0<<6)   // 0:PD3, 1:MISO1
	| (0<<4)   // 0:PD2, 1:MOSI1
	| (0<<2)   // 0:PD1, 1:SCK1
	| (0<<0)   // 0:PD0, 1:SS1
		;		
	PCD->CR = 0   // 0xFFFFFFFF(reset value) Input, output, open drain, analog
	| (0<<30)   // P15 0:Push-pull output, 1:Open-drain output, 2:Input, 3:Analog
	| (0<<28)   // P14
	| (0<<26)   // P13
	| (0<<24)   // P12
	| (0<<22)   // P11
	| (0<<20)   // P10
	| (0<<18)   // P9
	| (0<<16)   // P8
	| (0<<14)   // P7
	| (0<<12)   // P6
	| (1<<10)   // P5
	| (1<<8)   // P4
	| (0<<6)   // P3
	| (0<<4)   // P2
	| (0<<2)   // P1
	| (0<<0)   // P0	
		; 	
	PCD->PCR = 0 // pullup (0,1)
	| (0<<15)   // P15 0:Disable pull-up resistor, 1:Enable pull-up resister
	| (0<<14)   // P14
	| (0<<13)   // P13
	| (0<<12)   // P12
	| (0<<11)   // P11
	| (0<<10)   // P10
	| (0<<9)   // P9
	| (0<<8)   // P8
	| (0<<7)   // P7
	| (0<<6)   // P6
	| (0<<5)   // P5
	| (0<<4)   // P4
	| (0<<3)   // P3
	| (0<<2)   // P2
	| (0<<1)   // P1
	| (0<<0)   // P0	
		; 
	PORT_ACCESS_DIS();  // disable writing permittion of ALL PCU register 	
}
